import java.util.* ;
import java.math.* ;
/*
  QS

 Luc Maranget, projet Moscova, Inria Rocquencourt
 
 Copyright 2005 Institut National de Recherche en Informatique et    
 en Automatique.  All rights reserved.  This file is distributed     
 under the terms of the Q Public License version 1.0.          
*/
class Row {
  private BitSet me, up ;
  private static int ncols ;

  Row(int i, IntList me) {
    up = new BitSet() ; up.set(i) ;
    this.me = new BitSet() ;
    boolean odd = false ;
    int p = -1  ;
    while (me != null) {
      odd = true ;
      p = me.val ;
      me = me.next ;      
      for ( ; me != null && me.val == p ; me = me.next) {
        odd = !odd ;
      }
      if (odd) {
        if (p+1 > ncols) ncols = p+1 ;
        this.me.set(p) ;
      }
    }
  }

  private static void gauss(Row [] mat) {
    int m = mat.length ;
    for (int i = 0 ; i < ncols ; i++) {
      for (int ip = i ; ip < m ; ip++) {
        Row piv = mat[ip] ;
        if (piv.me.get(i)) { // Found pivot
          for (int k = ip+1 ; k < m ; k++) {
            Row row = mat[k] ;
            if (row.me.get(i)) { // Combine with pivot
              row.me.xor(piv.me) ;
              row.up.xor(piv.up) ;
            }
          }
          // SWAP i <-> ip
          mat[ip] = mat[i] ;
          mat[i] = piv ;
          break ;
        }
      }
    }
  }

  private static void invGauss(Row [] mat) {
    int m = mat.length ;
    for (int i = ncols-1 ; i >= 0 ; i--) {
      for (int ip = ncols-i-1 ; ip < m ; ip++) {
        Row piv = mat[ip] ;
        if (piv.me.get(i)) { // Found pivot
          for (int k = ip+1 ; k < m ; k++) {
            Row row = mat[k] ;
            if (row.me.get(i)) { // Combine with pivot
              row.me.xor(piv.me) ;
              row.up.xor(piv.up) ;
            }
          }
          // SWAP i <-> ip
          mat[ip] = mat[ncols-i-1] ;
          mat[ncols-i-1] = piv ;
          break ;
        }
      }
    }
  }

  private static boolean factor (BitSet up, Eqs eqs) {
    BigInteger X = BigInteger.ONE ;
    BigInteger Y = BigInteger.ONE ;

    for (int k = up.nextSetBit(0); k >= 0; k = up.nextSetBit(k+1)) {
      BigInteger xi = eqs.eq[k].x ;
      X = X.multiply(xi) ;
      if (eqs.eq[k].ps == null) {
        Y = Y.multiply(xi.multiply(xi).subtract(eqs.n)) ;
      } else { // In case of large prime variation..
        Y = Y.multiply(eqs.eq[k].ps.prod()) ;
      }
    }
    Y = Arith.sqrt(Y) ;
    BigInteger f = eqs.n.gcd(X.subtract(Y).abs()) ; //abs cannot hurt here
    if (f.compareTo(BigInteger.ONE) == 0 || f.compareTo(eqs.n) == 0) {
      return false ;
    } else {
      System.out.println( f + " * " + eqs.n.divide(f)) ;
      return true ;
    }
  }

  static void solve(Row [] mat, Eqs eqs) {
    invGauss(mat) ;
    for (int i = mat.length-1 ; i >= 0 ; i--) {
      if (mat[i].me.nextSetBit(0) < 0) { // mat[i].me is empty
        if (factor(mat[i].up, eqs)) return ;
      }
    }
    System.out.println( 1 + " * " + eqs.n) ;
  }
}
