import java.math.* ;
import java.io.* ;

/*
  QS

 Luc Maranget, projet Moscova, Inria Rocquencourt
 
 Copyright 2005 Institut National de Recherche en Informatique et    
 en Automatique.  All rights reserved.  This file is distributed     
 under the terms of the Q Public License version 1.0.          
*/

class Simple extends Sieve {
  Simple (int [] p, int [] r,  BigInteger n, int sz, int B, Collector col,
          Options o) {
    super(p, r, n, sz, B, col, o) ;
  }

  boolean zyva(BigInteger v, int id) {
    int len = base.length ;
    BigInteger m = poly(v) ;
    short log1 = (short)m.bitLength() ;
    short log2 = (short)poly(v.add(BigInteger.valueOf(sz))).bitLength()  ;
    this.v = v ;


    //  Initialize w, (notice that w[i] decreases during sieving !
    fillW((short)-(logB), w, 0, sz, log1, log2) ;
    if (verbose > 1) {
      dumpW("log" + id + ".ref") ;
    }
    int log = 1 ;
    int lim = 3 ;

    // Sieveing loop
    if (verbose > 1) System.err.println("Start sieving: " + base[2]) ;
    for (int i = 2  ;  i < len ; i++) {
      int p = base[i] ;

      // Compute log of factors incrementally
      while (p >= lim) {
        log++ ;
        long tlog = log ;
        long k = (long)1 << tlog ;
        double x = k * sqrt2 ;
        lim = (int)x ;
      }

      // Find starting point
      int r = root[i] ;
      int vmod = v.remainder(bbase[i]).intValue() ;
      int xi=0 ,yi = 0;

      if (vmod < r) {
        xi = r-vmod ;
      } else if (vmod > r) {
        xi = (r+p) - vmod ;
      }
      r = p-r ;
      if (vmod < r) {
        yi = r-vmod ;
      } else if (vmod > r) {
        yi = (r+p) - vmod ;
      }
      if (xi > yi) { int t = xi ; xi=yi ; yi=t ; }

      // Now sieve multiples of p
      int k = xi ;
      for (int kk=yi ; kk < sz ; k += p, kk += p) {
        w[k] -= log ;
        w[kk] -= log ;
      }
      if (k < sz)
        w[k] -= log ;
    }
  
  
    if (verbose > 1) {
      dumpW("log" + id + ".log") ;
    }

    // Check factorization
    for (int i = 0 ; i < sz ; i++) {
      if (w[i] <= 0) {
        if (tryFactorThis(i)) return true ;
      }
    }
    //    again() ;
    if (verbose > 0) efficiency(id) ;
    return false ;
  }

  private void dumpW(String name) {
    try {
      PrintWriter f = new PrintWriter (new FileWriter(name)) ;
      for (int i = 0 ; i < sz ; i++)
        f.println(i + " " + w[i]) ;
      f.close() ;
    } catch (Exception e) {
      
    }
  }
}
